import unittest
from traits.testing.api import doctest_for_module

import scimath.units.unit_scalar as unit_scalar
from scimath.units.unit_scalar import UnitScalar
from scimath.units.length import m
from scimath.units.unit import unit


from nose.tools import assert_equal, assert_is_instance

class UnitScalarDocTestCase(doctest_for_module(unit_scalar)):
    pass


def test_offset_unit_computations():
    """ Executing some basic computations with a basic custom unit with offset.
    """
    my_u = unit(12, m.derivation, 14)
    s1 = UnitScalar(3, units=my_u)
    s2 = UnitScalar(5, units=my_u)
    s3 = s1+s2
    assert_equal(s3,UnitScalar(8, units=my_u))



if __name__ == '__main__':
    import sys
    unittest.main(argv=sys.argv)



#!/usr/bin/python
r"""Modules with selected groups of units

The :mod:`~natu.groups.si` module includes all of the base and derived `SI
units`_.  The :mod:`~natu.groups.constants` module contains all constants
(physical quantities which are not units).  These modules contain all of the
constants and units with the indicated dimension:

- :mod:`~natu.groups.acceleration`
- :mod:`~natu.groups.amount`
- :mod:`~natu.groups.angle`
- :mod:`~natu.groups.area`
- :mod:`~natu.groups.charge`
- :mod:`~natu.groups.conductance`
- :mod:`~natu.groups.current`
- :mod:`~natu.groups.dimensionless`
- :mod:`~natu.groups.energy`
- :mod:`~natu.groups.force`
- :mod:`~natu.groups.frequency`
- :mod:`~natu.groups.length`
- :mod:`~natu.groups.magnetic_flux`
- :mod:`~natu.groups.magnetic_flux_density`
- :mod:`~natu.groups.mass`
- :mod:`~natu.groups.potential`
- :mod:`~natu.groups.power`
- :mod:`~natu.groups.pressure`
- :mod:`~natu.groups.resistance`
- :mod:`~natu.groups.temperature`
- :mod:`~natu.groups.time`
- :mod:`~natu.groups.velocity`
- :mod:`~natu.groups.volume`

These modules require that the `SI units`_ are defined in the selected INI_
files.  `BIPM.ini <BIPM-ini.html>`_ contains those definitions, and it is loaded
by default.

Each module can only be reloaded once.\ [#f1]_


.. _SI units: http://en.wikipedia.org/wiki/International_System_of_Units
.. _INI: http://en.wikipedia.org/wiki/INI_file

.. rubric:: Footnotes

.. [#f1] This is typical in Python.  From :func:`imp.reload`:

   "In many cases, however, extension modules are not designed to be initialized
   more than once, and may fail in arbitrary ways when reloaded."
"""
__author__ = "Kevin Davies"
__email__ = "kdavies4@gmail.com"
__copyright__ = ("Copyright 2013-2014, Kevin Davies, Hawaii Natural Energy "
                 "Institute, and Georgia Tech Research Corporation")
__license__ = "BSD-compatible (see LICENSE.txt)"

import sys

from textwrap import fill
from ..core import DimObject, UnitsModule, Unit

DOC_LINE_LENGTH = 74

def _update_module(name, units, dimension=None):
    """Update the module with name *name* (:class`str`) to contain the units in
    :class:`dict` *units*, optionally filtered to those with
    :class:`~natu.core.CompoundDimension` *dimension*.

    If *dimension* is *None*, all units are included.
    """
    # Retrieve the module.
    module = sys.modules[name]

    # Get the units and note the contents.
    if dimension is None:
        module.__doc__ += "\nContents:\n"
    else:
        units = {symbol: unit for symbol, unit in units.items()
                 if symbol != 'coherent_relations'
                 and isinstance(unit, DimObject)
                 and unit.dimension == dimension}
        module.__doc__ += "\nDefault contents:\n"
    module.__doc__ += fill(", ".join(sorted(units)), DOC_LINE_LENGTH)

    # Note the prefixable units, if any.
    prefixable = [symbol for symbol, unit in units.items()
                  if isinstance(unit, Unit) and unit.prefixable]
    if prefixable:
        module.__doc__ += "\n\nPrefixable subset:\n"
        module.__doc__ += fill(", ".join(sorted(prefixable)), DOC_LINE_LENGTH)

    # Update the module.
    sys.modules[name] = UnitsModule(module, units)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""General supporting functions

**Functions:**

- :func:`delayed_exit` - Exit with a message and a delay.

- :func:`flatten_list` - Flatten a nested list.

- :func:`format_e`: Format the scientific notation in a numeric string

- :func:`get_group` - Return a tuple with the contents of a parenthetical
  expression and everything after it.

- :func:`list_packages` - Return a list of the names of a module and its
  subpackages.

- :func:`multiglob` - Return a set of filenames that match sets of pathnames and
  extensions.

- :func:`num2super` - Convert a number to Unicode_ superscript.

- :func:`replace` - Perform a list of replacements on the text in a list of
  files.

- :func:`str2super` - Convert a numeric string to Unicode_ superscript.

- :func:`yes` - Ask a yes/no question and return *True* if the answer is 'Y' or
  'y'.


.. _Unicode: http://www.unicode.org/
"""
from __future__ import division, unicode_literals

__author__ = "Kevin Davies"
__email__ = "kdavies4@gmail.com"
__copyright__ = ("Copyright 2013-2014, Kevin Davies, Hawaii Natural Energy "
                 "Institute, and Georgia Tech Research Corporation")
__license__ = "BSD-compatible (see LICENSE.txt)"

# pylint: disable=I0011, C0103

# from functools import reduce # Valid in Python 2.6+, required in Python 3

import os
import re
import sys
import time

from glob import glob
from pkgutil import walk_packages

def delayed_exit(message="Exiting...", t=0.5):
    """Exit with a message (*message*) and a delay of *t* seconds.

    **Example:**

    >>> delayed_exit() # doctest: +SKIP
    Exiting...
    """
    print(message)
    time.sleep(t)
    exit()

def flatten_list(l, ltypes=(list, tuple)):
    """Flatten a nested list.

    **Parameters:**

    - *l*: List (may be nested to an arbitrary depth)

          If the type of *l* is not in ltypes, then it is placed in a list.

    - *ltypes*: Tuple (not list) of accepted indexable types

    **Example:**

    >>> flatten_list([1, [2, 3, [4]]])
    [1, 2, 3, 4]
    """
    # Based on
    # http://rightfootin.blogspot.com/2006/09/more-on-python-flatten.html,
    # 10/28/2011
    ltype = type(l)
    if ltype not in ltypes:  # So that strings aren't split into characters
        return [l]
    l = list(l)
    i = 0
    while i < len(l):
        while isinstance(l[i], ltypes):
            if l[i]:
                l[i:i + 1] = l[i]
            else:
                l.pop(i)
                i -= 1
                break
        i += 1
    return ltype(l)

def format_e(num_str, code):
    """Format the scientific notation in a numeric string

    **Parameters:**

    - *num_str*: A numeric string

         If the string does not contain 'e' (no scientific notation), then it is
         returned directly.

    - *code*: Format code

         'H' is for HTML, 'L' is for LaTeX_, and 'U' is for Unicode_.


    .. _LaTeX: http://www.latex-project.org/
    """
    # TODO: Add examples.
    num_str = num_str.replace('E', 'e')
    try:
        base, exp = num_str.split('e')
    except ValueError:
        pass
    else:
        exp = exp.lstrip('+')
        if exp[0] == '-':
           exp = '-' + exp[1:].lstrip('0')
        else:
            exp = exp.lstrip('0')
        if code == 'H':
            return base + '&times;10<sup>%s</sup>' % exp
        if code == 'L':
            return base + r' \times 10^{%s}' % exp
        if code == 'P':
            return base + r'✕10' + str2super(exp)
    return num_str

def get_group(expr):
    """Return 1) the contents of a parenthetical expression and 2) everything
    after it.

    The parenthetical expression starts at the beginning of *expr* and may be
    nested.

    **Example:**

    >>> get_group("((abc)*d)*2") # doctest: +SKIP
    ('(abc)*d', '*2')

    .. testcleanup::
       >>> assert get_group("((abc)*d)*2") == ('(abc)*d', '*2')
    """
    i = 0
    count = 0
    length = len(expr)
    while i < length:
        if expr[i] == '(':
            count += 1
        elif expr[i] == ')':
            count -= 1
        if count == 0:
            return expr[1:i], expr[i + 1:]
        i += 1


def list_packages(package):
    """Return a list of the names of a package and its subpackages.

    This only works if the package has a :attr:`__path__` attribute, which is
    not the case for some (all?) of the built-in packages.

    **Example:**

    >>> from natu import groups
    >>> for package in list_packages(groups):
    ...     print(package)
    natu.groups
    natu.groups.acceleration
    natu.groups.amount
    natu.groups.angle
    natu.groups.area
    natu.groups.charge
    natu.groups.conductance
    natu.groups.constants
    natu.groups.current
    natu.groups.dimensionless
    natu.groups.energy
    natu.groups.force
    natu.groups.frequency
    natu.groups.length
    natu.groups.magnetic_flux
    natu.groups.magnetic_flux_density
    natu.groups.mass
    natu.groups.potential
    natu.groups.power
    natu.groups.pressure
    natu.groups.resistance
    natu.groups.si
    natu.groups.temperature
    natu.groups.time
    natu.groups.velocity
    natu.groups.volume
    """
    # Based to unutbu's response at
    # http://stackoverflow.com/questions/1707709

    # pylint: disable=I0011, W0612

    names = [package.__name__]
    for __, name, __ in walk_packages(package.__path__,
                                      prefix=package.__name__ + '.',
                                      onerror=lambda x: None):
        names.append(name)
    return names


def multiglob(pathnames, extensions={'*.mat'}):
    r"""Return a set of filenames that match a pathname pattern.

    Unlike Python's :func:`glob.glob` function, this function runs an additional
    expansion on matches that are directories.

    **Parameters:**

    - *pathnames*: String or set of strings used to match files or folders

         This may contain Unix shell-style patterns:

         ============   ============================
         Character(s)   Role
         ============   ============================
         \*             Matches everything
         ?              Matches any single character
         [seq]          Matches any character in seq
         [!seq]         Matches any char not in seq
         ============   ============================

   - *extensions*: Filename pattern or set of patterns that should be used to
     match files in any directories generated from *pathnames*

        These may also use the shell-style patterns above.
    """
    # TODO: Add an example.
    # **Example:**
    # >>> multiglob("examples/ChuaCircuit/*/") # doctest: +SKIP
    # ['examples/ChuaCircuit/1/dsres.mat', 'examples/ChuaCircuit/2/dsres.mat']
    # .. testcleanup::
    #    >>> sorted(multiglob("examples/ChuaCircuit/*/"))
    #    ['examples/ChuaCircuit/1/dsres.mat', 'examples/ChuaCircuit/2/dsres.mat']

    # Since order is arbitrary, the doctest is skipped here and included in
    # tests/tests.txt instead.
    fnames = set()
    for pathname in flatten_list(pathnames):
        items = glob(pathname)
        for item in items:
            if os.path.isdir(item):
                for ext in set(extensions):
                    fnames = fnames.union(glob(os.path.join(item, ext)))
            else:
                fnames.add(item)
    return fnames


# Exponential formatting
# Based on pint.formatting (https://github.com/hgrecco/pint, 7/11/14):
# - Copyright 2013 by Pint Authors
# - BSD license

def num2super(num):
    """Convert a number to Unicode_ superscript.
    """
    assert num % 1 == 0, "Only whole numbers are supported."
    # As of 7/16/14, there is no Unicode decimal or division sign.
    return str2super('%i' % num)

# In Python3, can do this (simpler and probably faster):
# EXP_TRANS = str.maketrans('0123456789', _UNICODE_EXPS)
# def num2super(num):
#     """Return a number as a Unicode superscript.
#     """
#     assert num % 1 == 0, "Only whole numbers are supported."
#     return ('%i' % num).replace('-', '⁻').translate(EXP_TRANS)

_UNICODE_EXPS = '⁰¹²³⁴⁵⁶⁷⁸⁹'

def str2super(num_str):
    """Convert a numeric string to Unicode_ superscript.
    """
    exp_str = num_str.replace('-', '⁻')
    for n in range(10):
        exp_str = exp_str.replace(str(n), _UNICODE_EXPS[n])
    return exp_str


def replace(fnames, rpls):
    r"""Perform a list of replacements on the text in a list of files.

    **Parameters:**

    - *fnames*: Filename or list of filenames

    - *rpl*: List of replacements to make

         Each entry is a tuple of (*src*, *dest*), where *dest* is a string that
         will replace the *src* string.  Each string can use Python's :mod:`re`
         expressions; see https://docs.python.org/2/library/re.html for details.

    **Example:**

    >>> fname = 'temp.txt'
    >>> with open(fname, 'w') as f:
    ...     __ = f.write("apple orange banana")
    >>> replace([fname], [('ba(.*)', r'ba\1\1')])
    >>> with open(fname, 'r') as f:
    ...     print(f.read())
    apple orange banananana

    .. testcleanup::
       >>> os.remove(fname)
    """

    # Compile the regular expressions.
    for i, (old, new) in enumerate(rpls):
        rpls[i] = (re.compile(old), new)

    # Read each file and make the replacements.
    for fname in flatten_list(fnames):
        with open(fname, 'r+') as f:
            text = f.read()
            for (old, new) in rpls:
                text = old.sub(new, text)
            f.seek(0)
            f.write(text)
            f.truncate()

# Getch classes based on
# http://code.activestate.com/recipes/134892-getch-like-unbuffered-character-reading-from-stdin/,
# accessed 5/31/14

class _Getch(object):

    """Get a single character from the standard input.
    """

    def __init__(self):
        try:
            self.impl = _GetchWindows()
        except ImportError:
            self.impl = _GetchUnix()

    def __call__(self):
        return self.impl()


class _GetchUnix(object):

    """Get a single character from the standard input on Unix.
    """

    def __init__(self):
        # pylint: disable=I0011, W0611, W0612
        import tty

    def __call__(self):
        import tty
        import termios
        fd = sys.stdin.fileno()
        old_settings = termios.tcgetattr(fd)
        try:
            tty.setraw(sys.stdin.fileno())
            ch = sys.stdin.read(1)
        finally:
            termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
        return ch


class _GetchWindows(object):

    """Get a single character from the standard input on Windows.
    """
    # pylint: disable=I0011, W0611, W0612

    def __init__(self):
        import msvcrt

    def __call__(self):
        import msvcrt
        return msvcrt.getch()


def yes(question):
    """Ask a yes/no question and return *True* if the answer is 'Y' or 'y'.

    **Parameters:**

    - *question*: String representing the question to the user

    **Example:**

    >>> if yes("Do you want to print hello (y/n)?"): # doctest: +SKIP
    ...     print("hello")
    """
    getch = _Getch()
    sys.stdout.write(question + ' ')
    answer = getch()
    print(answer)
    return answer.lower() == 'y'



#!/usr/bin/python
"""Top-level test suite for natu
"""

# pylint: disable=I0011, C0111, R0904, W0621

import unittest
import doctest
import natu as thismodule

from glob import glob
from natu.util import list_packages
from os import path

THIS_DIR = path.dirname(path.realpath(__file__))
DOCTEST_FILES = [path.basename(fname)
                 for fname in glob(path.join(THIS_DIR, '*.txt'))]
PACKAGE_NAMES = list_packages(thismodule)

# TODO: Test with complex and Fraction from Python standard library

class Tests(unittest.TestCase):

    """Main set of unit tests
    """

    def test_equal(self):
        self.assertEqual(5, 5)


def test_suite():
    """Return a suite of all the tests.
    """
    print("Note: The IPython notebooks (examples/*.ipynb) must be tested "
          "manually.")
    suite = unittest.TestSuite()
    suite.addTests([unittest.makeSuite(Tests)])
    suite.addTests([doctest.DocFileSuite(fname) for fname in DOCTEST_FILES])
    suite.addTests([doctest.DocTestSuite(package)
                    for package in PACKAGE_NAMES])
    return suite

if __name__ == '__main__':
    # Run the tests here.

    RUNNER = unittest.TextTestRunner()
    RUNNER.run(test_suite())

#
"""
  // ----------------------------------------------------------------------
  // Set 1:  Mathematical constants and relations
  test(pi/3.14159265358979323846264338327950288419716939937510, name=
    "1 in set 1");
  // Value from http://en.wikipedia.org/wiki/Pi#Approximate_value
  test(cyc/(2*pi*rad), name="3 in set 1");
  test(cyc/(360*degree), name="4 in set 1");
  test('%'/0.01, name="5 in set 1");
  // ----------------------------------------------------------------------
  // Set 2:  Relations from [BIPM2006]
  test(q/(1.602176487e-19*C), name="1 in set 2");
  test(C*V/J, name="2 in set 2");
  // ----------------------------------------------------------------------
  // Set 3: Coherent derived units in the SI with special names and
  // symbols [BIPM2006]
  test(sr/rad^2, name="1 in set 3");
  // See note below the definition of sr in FCSys.Units.
  test(Hz/(cyc/s), name="2 in set 3");
  // See note below the definition of Hz in FCSys.Units re: inclusion of cyc.
  test(N/(kg*m/s^2), name="3 in set 3");
  test(Pa/(N/m^2), name="4 in set 3");
  test(J/(N*m), name="5 in set 3");
  test(W/(J/s), name="6 in set 3");
  test(C/(A*s), name="7 in set 3");
  test(V/(W/A), name="8 in set 3");
  test(F/(C/V), name="9 in set 3");
  test(ohm/(V/A), name="10 in set 3");
  test(S/(A/V), name="11 in set 3");
  test(Wb*Hz/V, name="12 in set 3");
  // See note below the definition of V in FCSys.Units re: Hz instead of 1/s.
  test(T/(Wb/m^2), name="13 in set 3");
  test(H/(V*s/A), name="14 in set 3");
  test(lm/(cd*sr), name="15 in set 3");
  test(lx/(lm/m^2), name="16 in set 3");
  test(Gy/(J/kg), name="17 in set 3");
  test(kat/(mol/s), name="18 in set 3");
  // ----------------------------------------------------------------------
  // Set 4:  Relations from [NIST2010]
  // Generated from Resources/NIST.xls, 2013-1-23
  test(1/alpha/137.035999074, name="inverse fine-structure constant");
  test(1/G_0/(12906.4037217*ohm), name="inverse of conductance quantum");
  test(cyc/m*h*c/(1.239841930e-6*eV), name=
    "inverse meter-electron volt relationship");
  // See note below the definition of m in FCSys.Units re: inclusion of cyc.
  test(cyc/m*h*c/(4.556335252755e-8*E_h), name=
    "inverse meter-hartree relationship");
  // See note below the definition of m in FCSys.Units re: inclusion of cyc.
  test(cyc/m*h*c/(1.986445684e-25*J), name=
    "inverse meter-joule relationship");
  // See note below the definition of m in FCSys.Units re: inclusion of cyc.
  test(cyc/m*h*c/k_B/(1.4387770e-2*K), name=
    "inverse meter-kelvin relationship");
  // See note below the definition of m in FCSys.Units re: inclusion of cyc.
  test(cyc/m*h/c/(2.210218902e-42*kg), name=
    "inverse meter-kilogram relationship");
  // See note below the definition of m in FCSys.Units re: inclusion of cyc.
  test(100*kPa/(k_B*273.15*K)/(2.6516462e25/m^3), name=
    "Loschmidt constant (273.15 K, 100 kPa)");
  test(101.325*kPa/(k_B*273.15*K)/(2.6867805e25/m^3), name=
    "Loschmidt constant (273.15 K, 101.325 kPa)");
  test(12*g/mol/(12e-3*kg/mol), name="molar mass of carbon-12");
  test(alpha/7.2973525698e-3, name="fine-structure constant");
  test(atm/(101325*Pa), name="standard atmosphere");
  test(c/(299792458*m/s), name="speed of light in vacuum");
  test(c_1/(3.74177153e-16*W*m^2), name="first radiation constant");
  test(c_2*cyc/(1.4387770e-2*m*K), name="second radiation constant");
  // See note below the definition of m in FCSys.Units re: inclusion of cyc.
  test(c_3_f/(5.8789254e10*Hz/K), name=
    "Wien frequency displacement law constant");
  test(c_3_lambda/(2.8977721e-3*K*m/cyc), name=
    "Wien wavelength displacement law constant");
  // See note below the definition of m in FCSys.Units re: inclusion of cyc.
  test(cyc/m*c/(299792458*Hz), name="inverse meter-hertz relationship");
  // See note below the definition of m in FCSys.Units re: inclusion of cyc.
  test(E_h/(4.35974434e-18*J), name="Hartree energy");
  test(E_h/(27.21138505*eV), name="Hartree energy in eV");
  test(E_h/(27.21138505*eV), name="hartree-electron volt relationship");
  test(E_h/(4.35974434e-18*J), name="hartree-joule relationship");
  test(E_h/(h*c)/(2.194746313708e7*cyc/m), name=
    "hartree-inverse meter relationship");
  // See note below the definition of m in FCSys.Units re: inclusion of cyc.
  test(E_h/c^2/(4.85086979e-35*kg), name="hartree-kilogram relationship");
  test(E_h/h/(6.579683920729e15*Hz), name="hartree-hertz relationship");
  test(E_h/k_B/(3.1577504e5*K), name="hartree-kelvin relationship");
  test(epsilon_0/(8.854187817e-12*F/m), name="electric constant");
  test(eV/(1.602176565e-19*J), name="electron volt");
  test(eV/(3.674932379e-2*E_h), name="electron volt-hartree relationship");
  test(eV/(1.602176565e-19*J), name="electron volt-joule relationship");
  test(eV/(h*c)/(8.06554429e5*cyc/m), name=
    "electron volt-inverse meter relationship");
  // See note below the definition of m in FCSys.Units re: inclusion of cyc.
  test(eV/c^2/(1.782661845e-36*kg), name=
    "electron volt-kilogram relationship");
  test(eV/h/(2.417989348e14*Hz), name="electron volt-hertz relationship");
  test(eV/k_B/(1.1604519e4*K), name="electron volt-kelvin relationship");
  test(G_0/(7.7480917346e-5*S), name="conductance quantum");
  test(g/mol/(1e-3*kg/mol), name="molar mass constant");
  test(h/(6.62606957e-34*J/Hz), name="Planck constant");
  // See note below the definition of h in FCSys.Units re: the usage of 1/Hz
  // instead of s.
  test(h/(4.135667516e-15*eV/Hz), name="Planck constant in eV s");
  // See note below the definition of h in FCSys.Units re: the usage of 1/Hz
  // instead of s.
  test(h*c/(2*pi)/(197.3269718*mega*eV*femto*m/cyc), name=
    "Planck constant over 2 pi times c in MeV fm");
  // See note below the definition of m in FCSys.Units re: inclusion of cyc.
  test(h/(2*pi)/(1.054571726e-34*J/Hz), name="Planck constant over 2 pi");
  // See note below the definition of h in FCSys.Units re: the usage of 1/Hz
  // instead of s.
  test(h/(2*pi)/(6.58211928e-16*eV/Hz), name=
    "Planck constant over 2 pi in eV s");
  // See note below the definition of h in FCSys.Units re: the usage of 1/Hz
  // instead of s.
  test(Hz*h/(4.135667516e-15*eV), name="hertz-electron volt relationship");
  test(Hz*h/(1.5198298460045e-16*E_h), name="hertz-hartree relationship");
  test(Hz*h/(6.62606957e-34*J), name="hertz-joule relationship");
  test(Hz*h/c^2/(7.37249668e-51*kg), name="hertz-kilogram relationship");
  test(Hz*h/k_B/(4.7992434e-11*K), name="hertz-kelvin relationship");
  test(Hz/c/(3.335640951e-9*cyc/m), name="hertz-inverse meter relationship");
  // See note below the definition of m in FCSys.Units re: inclusion of cyc.
  test(J/(6.24150934e18*eV), name="joule-electron volt relationship");
  test(J/(2.29371248e17*E_h), name="joule-hartree relationship");
  test(J/(h*c)/(5.03411701e24*cyc/m), name=
    "joule-inverse meter relationship");
  // See note below the definition of m in FCSys.Units re: inclusion of cyc.
  test(J/c^2/(1.112650056e-17*kg), name="joule-kilogram relationship");
  test(J/h/(1.509190311e33*Hz), name="joule-hertz relationship");
  test(J/k_B/(7.2429716e22*K), name="joule-kelvin relationship");
  test(k_B/(1.3806488e-23*J/K), name="Boltzmann constant");
  test(k_B/(8.6173324e-5*eV/K), name="Boltzmann constant in eV/K");
  test(k_B/(h*c)/(69.503476*cyc/(m*K)), name=
    "Boltzmann constant in inverse meters per kelvin");
  // See note below the definition of m in FCSys.Units re: inclusion of cyc.
  test(k_B/h/(2.0836618e10*Hz/K), name="Boltzmann constant in Hz/K");
  test(k_F/(96485.3365*C/mol), name="Faraday constant");
  test(K*k_B/(8.6173324e-5*eV), name="kelvin-electron volt relationship");
  test(K*k_B/(3.1668114e-6*E_h), name="kelvin-hartree relationship");
  test(K*k_B/(1.3806488e-23*J), name="kelvin-joule relationship");
  test(K*k_B/(h*c)/(69.503476*cyc/m), name=
    "kelvin-inverse meter relationship");
  // See note below the definition of m in FCSys.Units re: inclusion of cyc.
  test(K*k_B/c^2/(1.5361790e-40*kg), name="kelvin-kilogram relationship");
  test(K*k_B/h/(2.0836618e10*Hz), name="kelvin-hertz relationship");
  test(kg*c^2/(5.60958885e35*eV), name=
    "kilogram-electron volt relationship");
  test(kg*c^2/(2.061485968e34*E_h), name="kilogram-hartree relationship");
  test(kg*c^2/(8.987551787e16*J), name="kilogram-joule relationship");
  test(kg*c^2/h/(1.356392608e50*Hz), name="kilogram-hertz relationship");
  test(kg*c^2/k_B/(6.5096582e39*K), name="kilogram-kelvin relationship");
  test(kg*c/h/(4.52443873e41*cyc/m), name=
    "kilogram-inverse meter relationship");
  // See note below the definition of m in FCSys.Units re: inclusion of cyc.
  test(mu_0/(12.566370614e-7*N/A^2), name="mag. constant");
  test(N_A/(6.02214129e23/mol), name="Avogadro constant");
  test(N_A*h/(3.9903127176e-10*J/(Hz*mol)), name="molar Planck constant");
  // See note below the definition of h in FCSys.Units re: the usage of 1/Hz
  // instead of s.
  test(N_A*h*c/(0.119626565779*J*m/(cyc*mol)), name=
    "molar Planck constant times c");
  // See note below the definition of m in FCSys.Units re: inclusion of cyc.
  test(Phi_0/(2.067833758e-15*Wb), name="mag. flux quantum");
  test(q/(1.602176565e-19*C), name="elementary charge");
  test(q/h/(2.417989348e14*Hz/V), name="elementary charge over h");
  // See note below the definition of h in FCSys.Units re: the usage of Hz/V
  // instead of A/J.
  test(R/(8.3144621*J/(mol*K)), name="molar gas constant");
  test(R_inf/(10973731.568539*cyc/m), name="Rydberg constant");
  // See note below the definition of m in FCSys.Units re: inclusion of cyc.
  test(R_inf*c/(3.289841960364e15*Hz), name=
    "Rydberg constant times c in Hz");
  test(R_inf*h*c/(13.60569253*eV), name="Rydberg constant times hc in eV");
  test(R_inf*h*c/(2.179872171e-18*J), name="Rydberg constant times hc in J");
  test(R*273.15*K/(100*kPa)/(22.710953e-3*m^3/mol), name=
    "molar volume of ideal gas (273.15 K, 100 kPa)");
  test(R*273.15*K/(101.325*kPa)/(22.413968e-3*m^3/mol), name=
    "molar volume of ideal gas (273.15 K, 101.325 kPa)");
  test(sigma/(5.670373e-8*W/(m^2*K^4)), name="Stefan-Boltzmann constant");
  test(Z_0/(376.730313461*ohm), name="characteristic impedance of vacuum");
"""



import math

from unittest import TestCase

from scimath.physical_quantities.dimensions import Dimensions, Dim

from scimath.physical_quantities.units import Unit, MultiplicativeUnit, \
    NamedUnit, BaseUnit

class UnitClassTest(TestCase):
    def setUp(self):
        self.length = Dimensions({"length": 1.0})
        self.mass = Dimensions({"mass": 1.0})
        self.time = Dimensions({"time": 1.0})
        self.temperature = Dimensions({"temperature": 1.0})
        self.force = self.mass*self.length/self.time**2
        self.power = self.force*self.length

        self.newton = Unit(symbol="N", expression="N", latex="\mathrm{N}",
                      dimensions=self.force)

        self.watt = Unit(symbol="W", expression="W", latex="\mathrm{W}",
                         dimensions=self.power)
        self.milliwatt = Unit(symbol="mW", expression="mW", latex="\mathrm{mW}",
                         scale=1000.0, dimensions=self.power)
        self.decibel = Unit(symbol="dB", expression="decibel",
                            latex="\mathrm{dB}", dimensions=self.power,
                            logarithmic=True, scale=10.0, offset=-3.0)
        self.neper = Unit(symbol="np", expression="neper",
                            latex="\mathrm{np}", dimensions=self.power,
                            logarithmic=True, scale=0.5, offset=-math.log(1000),
                            log_base=math.exp(1.0))

        self.kelvin = Unit(symbol="K", expression="K", latex="\mathrm{K}",
                      dimensions=self.temperature)
        self.also_kelvin = Unit(symbol="K", expression="K", latex="\mathrm{K}",
                      dimensions=self.temperature)
        self.kelvin_named = NamedUnit(symbol="K", expression="K", latex="\mathrm{K}",
                      dimensions=self.temperature)
        self.millikelvin = Unit(symbol="mK", expression="mK", latex="\mathrm{mK}",
                      dimensions=self.temperature, scale=1000.0)
        self.millikelvin_named = NamedUnit(symbol="mK", expression="mK", latex="\mathrm{mK}",
                      dimensions=self.temperature, scale=1000.0)
        self.kelvin_base = BaseUnit(symbol="K", expression="K", latex="\mathrm{K}",
                      dimensions=self.temperature)
        self.rankine = Unit(symbol="R", expression="R", latex="\mathrm{R}",
                      dimensions=self.temperature, scale=9.0/5.0)
        self.rankine_named = NamedUnit(symbol="R", expression="R", latex="\mathrm{R}",
                      dimensions=self.temperature, scale=9.0/5.0)
        self.degree_C = Unit(symbol=u"\u00B0C", expression="degree_C",
                       latex="\,^{\circ}\mathrm{C}", dimensions=self.temperature,
                       offset=273.15)
        self.degree_F = Unit(symbol=u"\u00B0F", expression="degree_F",
                       latex="\,^{\circ}\mathrm{F}", dimensions=self.temperature,
                       scale=9.0/5.0, offset=459.67*5.0/9.0)

    # Test equality
    def test_equality_simple_true(self):
        assert self.kelvin == self.also_kelvin

    def test_equality_simple_false(self):
        assert self.kelvin != self.newton

    def test_equality_base_simple_true(self):
        assert self.kelvin_base == self.kelvin

    # Test convert_to_base method
    def test_convert_to_base_logarithmic(self):
        self.assertAlmostEqual(self.decibel.convert_to_base(30.0), 1.0)
        self.assertAlmostEqual(self.decibel.convert_to_base(0.0), 0.001)

    def test_convert_to_base_logarithmic_natural(self):
        self.assertAlmostEqual(self.neper.convert_to_base(3.4538776394910684), 1.0)
        self.assertAlmostEqual(self.neper.convert_to_base(0.0), 0.001)

    # Automatically generated test cases to cover all cases of non-logarithmic
    # conversions
    #    Generated by the following code:
    """
def values(T):
    return {
        "degree_C": repr(T),
        "kelvin": repr(T+273.15),
        "millikelvin": repr((T+273.15)*1000.0),
        "millikelvin_named": repr((T+273.15)*1000.0),
        "kelvin_base": repr(T+273.15),
        "kelvin_named": repr(T+273.15),
        "degree_F": repr(T*9.0/5.0 + 32),
        "rankine": repr((T+273.15)*9.0/5.0),
        "rankine_named": repr((T+273.15)*9.0/5.0),
    }

def write_base_converter_test(unit, rows):
    func_code = "    def test_convert_to_base_%s(self):\n" % unit
    for row in rows:
        func_code += "        self.assertAlmostEqual(self.%s.convert_to_base(%s), %s)\n" % \
                     (unit, row[unit], row["kelvin"])
    func_code += "\n"
    func_code += "    def test_convert_from_base_%s(self):\n" % unit
    for row in rows:
        func_code += "        self.assertAlmostEqual(self.%s.convert_from_base(%s), %s)\n" % \
                     (unit, row["kelvin"], row[unit])
    func_code += "\n"
    return func_code


def write_complex_convert_test(from_unit, to_unit, rows):
    func_code = "    def test_convert_to_unit_%s_to_%s(self):\n" % \
                (from_unit, to_unit)
    for row in rows:
        func_code += "        self.assertAlmostEqual(self.%s.convert_to_unit(%s, self.%s), %s)\n" % \
                     (from_unit, row[from_unit], to_unit, row[to_unit])

    func_code += "\n"
    func_code = "    def test_convert_from_unit_%s_from_%s(self):\n" % \
                (from_unit, to_unit)
    for row in rows:
        func_code += "        self.assertAlmostEqual(self.%s.convert_from_unit(%s, self.%s), %s)\n" % \
                     (to_unit, row[from_unit], from_unit, row[to_unit])

    func_code += "\n"
    func_code += "    def test_make_converter_%s_from_%s(self):\n" % \
                (from_unit, to_unit)
    func_code += "        converter = self.%s.make_converter(self.%s)\n" % \
                 (from_unit, to_unit)
    for row in rows:
        func_code += "        self.assertAlmostEqual(converter(%s), %s)\n" % \
                     (row[from_unit], row[to_unit])

    func_code += "\n"
    return func_code

rows = [values(T) for T in [-273.15, -17.777777777777779, 0.0, 100.0]]
units = rows[0].keys()

for unit in units:
    print write_base_converter_test(unit, rows)

for from_unit in units:
    for to_unit in units:
        print write_complex_convert_test(from_unit, to_unit, rows)
    """

    def test_convert_to_base_degree_F(self):
        self.assertAlmostEqual(self.degree_F.convert_to_base(-459.66999999999996), 0.0)
        self.assertAlmostEqual(self.degree_F.convert_to_base(0.0), 255.37222222222221)
        self.assertAlmostEqual(self.degree_F.convert_to_base(32.0), 273.14999999999998)
        self.assertAlmostEqual(self.degree_F.convert_to_base(212.0), 373.14999999999998)

    def test_convert_from_base_degree_F(self):
        self.assertAlmostEqual(self.degree_F.convert_from_base(0.0), -459.66999999999996)
        self.assertAlmostEqual(self.degree_F.convert_from_base(255.37222222222221), 0.0)
        self.assertAlmostEqual(self.degree_F.convert_from_base(273.14999999999998), 32.0)
        self.assertAlmostEqual(self.degree_F.convert_from_base(373.14999999999998), 212.0)


    def test_convert_to_base_kelvin_base(self):
        self.assertAlmostEqual(self.kelvin_base.convert_to_base(0.0), 0.0)
        self.assertAlmostEqual(self.kelvin_base.convert_to_base(255.37222222222221), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_base.convert_to_base(273.14999999999998), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_base.convert_to_base(373.14999999999998), 373.14999999999998)

    def test_convert_from_base_kelvin_base(self):
        self.assertAlmostEqual(self.kelvin_base.convert_from_base(0.0), 0.0)
        self.assertAlmostEqual(self.kelvin_base.convert_from_base(255.37222222222221), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_base.convert_from_base(273.14999999999998), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_base.convert_from_base(373.14999999999998), 373.14999999999998)


    def test_convert_to_base_rankine(self):
        self.assertAlmostEqual(self.rankine.convert_to_base(0.0), 0.0)
        self.assertAlmostEqual(self.rankine.convert_to_base(459.66999999999996), 255.37222222222221)
        self.assertAlmostEqual(self.rankine.convert_to_base(491.66999999999996), 273.14999999999998)
        self.assertAlmostEqual(self.rankine.convert_to_base(671.66999999999996), 373.14999999999998)

    def test_convert_from_base_rankine(self):
        self.assertAlmostEqual(self.rankine.convert_from_base(0.0), 0.0)
        self.assertAlmostEqual(self.rankine.convert_from_base(255.37222222222221), 459.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_base(273.14999999999998), 491.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_base(373.14999999999998), 671.66999999999996)


    def test_convert_to_base_kelvin(self):
        self.assertAlmostEqual(self.kelvin.convert_to_base(0.0), 0.0)
        self.assertAlmostEqual(self.kelvin.convert_to_base(255.37222222222221), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin.convert_to_base(273.14999999999998), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin.convert_to_base(373.14999999999998), 373.14999999999998)

    def test_convert_from_base_kelvin(self):
        self.assertAlmostEqual(self.kelvin.convert_from_base(0.0), 0.0)
        self.assertAlmostEqual(self.kelvin.convert_from_base(255.37222222222221), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin.convert_from_base(273.14999999999998), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin.convert_from_base(373.14999999999998), 373.14999999999998)


    def test_convert_to_base_millikelvin_named(self):
        self.assertAlmostEqual(self.millikelvin_named.convert_to_base(0.0), 0.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_to_base(255372.22222222222), 255.37222222222221)
        self.assertAlmostEqual(self.millikelvin_named.convert_to_base(273150.0), 273.14999999999998)
        self.assertAlmostEqual(self.millikelvin_named.convert_to_base(373150.0), 373.14999999999998)

    def test_convert_from_base_millikelvin_named(self):
        self.assertAlmostEqual(self.millikelvin_named.convert_from_base(0.0), 0.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_base(255.37222222222221), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_base(273.14999999999998), 273150.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_base(373.14999999999998), 373150.0)


    def test_convert_to_base_millikelvin(self):
        self.assertAlmostEqual(self.millikelvin.convert_to_base(0.0), 0.0)
        self.assertAlmostEqual(self.millikelvin.convert_to_base(255372.22222222222), 255.37222222222221)
        self.assertAlmostEqual(self.millikelvin.convert_to_base(273150.0), 273.14999999999998)
        self.assertAlmostEqual(self.millikelvin.convert_to_base(373150.0), 373.14999999999998)

    def test_convert_from_base_millikelvin(self):
        self.assertAlmostEqual(self.millikelvin.convert_from_base(0.0), 0.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_base(255.37222222222221), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin.convert_from_base(273.14999999999998), 273150.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_base(373.14999999999998), 373150.0)


    def test_convert_to_base_degree_C(self):
        self.assertAlmostEqual(self.degree_C.convert_to_base(-273.14999999999998), 0.0)
        self.assertAlmostEqual(self.degree_C.convert_to_base(-17.777777777777779), 255.37222222222221)
        self.assertAlmostEqual(self.degree_C.convert_to_base(0.0), 273.14999999999998)
        self.assertAlmostEqual(self.degree_C.convert_to_base(100.0), 373.14999999999998)

    def test_convert_from_base_degree_C(self):
        self.assertAlmostEqual(self.degree_C.convert_from_base(0.0), -273.14999999999998)
        self.assertAlmostEqual(self.degree_C.convert_from_base(255.37222222222221), -17.777777777777779)
        self.assertAlmostEqual(self.degree_C.convert_from_base(273.14999999999998), 0.0)
        self.assertAlmostEqual(self.degree_C.convert_from_base(373.14999999999998), 100.0)


    def test_convert_to_base_rankine_named(self):
        self.assertAlmostEqual(self.rankine_named.convert_to_base(0.0), 0.0)
        self.assertAlmostEqual(self.rankine_named.convert_to_base(459.66999999999996), 255.37222222222221)
        self.assertAlmostEqual(self.rankine_named.convert_to_base(491.66999999999996), 273.14999999999998)
        self.assertAlmostEqual(self.rankine_named.convert_to_base(671.66999999999996), 373.14999999999998)

    def test_convert_from_base_rankine_named(self):
        self.assertAlmostEqual(self.rankine_named.convert_from_base(0.0), 0.0)
        self.assertAlmostEqual(self.rankine_named.convert_from_base(255.37222222222221), 459.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_base(273.14999999999998), 491.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_base(373.14999999999998), 671.66999999999996)


    def test_convert_to_base_kelvin_named(self):
        self.assertAlmostEqual(self.kelvin_named.convert_to_base(0.0), 0.0)
        self.assertAlmostEqual(self.kelvin_named.convert_to_base(255.37222222222221), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_named.convert_to_base(273.14999999999998), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_named.convert_to_base(373.14999999999998), 373.14999999999998)

    def test_convert_from_base_kelvin_named(self):
        self.assertAlmostEqual(self.kelvin_named.convert_from_base(0.0), 0.0)
        self.assertAlmostEqual(self.kelvin_named.convert_from_base(255.37222222222221), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_named.convert_from_base(273.14999999999998), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_named.convert_from_base(373.14999999999998), 373.14999999999998)


    def test_convert_from_unit_degree_F_from_degree_F(self):
        self.assertAlmostEqual(self.degree_F.convert_from_unit(-459.66999999999996, self.degree_F), -459.66999999999996)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(0.0, self.degree_F), 0.0)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(32.0, self.degree_F), 32.0)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(212.0, self.degree_F), 212.0)

    def test_make_converter_degree_F_from_degree_F(self):
        converter = self.degree_F.make_converter(self.degree_F)
        self.assertAlmostEqual(converter(-459.66999999999996), -459.66999999999996)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(32.0), 32.0)
        self.assertAlmostEqual(converter(212.0), 212.0)


    def test_convert_from_unit_degree_F_from_kelvin_base(self):
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(-459.66999999999996, self.degree_F), 0.0)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(0.0, self.degree_F), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(32.0, self.degree_F), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(212.0, self.degree_F), 373.14999999999998)

    def test_make_converter_degree_F_from_kelvin_base(self):
        converter = self.degree_F.make_converter(self.kelvin_base)
        self.assertAlmostEqual(converter(-459.66999999999996), 0.0)
        self.assertAlmostEqual(converter(0.0), 255.37222222222221)
        self.assertAlmostEqual(converter(32.0), 273.14999999999998)
        self.assertAlmostEqual(converter(212.0), 373.14999999999998)


    def test_convert_from_unit_degree_F_from_rankine(self):
        self.assertAlmostEqual(self.rankine.convert_from_unit(-459.66999999999996, self.degree_F), 0.0)
        self.assertAlmostEqual(self.rankine.convert_from_unit(0.0, self.degree_F), 459.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_unit(32.0, self.degree_F), 491.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_unit(212.0, self.degree_F), 671.66999999999996)

    def test_make_converter_degree_F_from_rankine(self):
        converter = self.degree_F.make_converter(self.rankine)
        self.assertAlmostEqual(converter(-459.66999999999996), 0.0)
        self.assertAlmostEqual(converter(0.0), 459.66999999999996)
        self.assertAlmostEqual(converter(32.0), 491.66999999999996)
        self.assertAlmostEqual(converter(212.0), 671.66999999999996)


    def test_convert_from_unit_degree_F_from_kelvin(self):
        self.assertAlmostEqual(self.kelvin.convert_from_unit(-459.66999999999996, self.degree_F), 0.0)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(0.0, self.degree_F), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(32.0, self.degree_F), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(212.0, self.degree_F), 373.14999999999998)

    def test_make_converter_degree_F_from_kelvin(self):
        converter = self.degree_F.make_converter(self.kelvin)
        self.assertAlmostEqual(converter(-459.66999999999996), 0.0)
        self.assertAlmostEqual(converter(0.0), 255.37222222222221)
        self.assertAlmostEqual(converter(32.0), 273.14999999999998)
        self.assertAlmostEqual(converter(212.0), 373.14999999999998)


    def test_convert_from_unit_degree_F_from_millikelvin_named(self):
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(-459.66999999999996, self.degree_F), 0.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(0.0, self.degree_F), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(32.0, self.degree_F), 273150.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(212.0, self.degree_F), 373150.0)

    def test_make_converter_degree_F_from_millikelvin_named(self):
        converter = self.degree_F.make_converter(self.millikelvin_named)
        self.assertAlmostEqual(converter(-459.66999999999996), 0.0)
        self.assertAlmostEqual(converter(0.0), 255372.22222222222)
        self.assertAlmostEqual(converter(32.0), 273150.0)
        self.assertAlmostEqual(converter(212.0), 373150.0)


    def test_convert_from_unit_degree_F_from_millikelvin(self):
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(-459.66999999999996, self.degree_F), 0.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(0.0, self.degree_F), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(32.0, self.degree_F), 273150.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(212.0, self.degree_F), 373150.0)

    def test_make_converter_degree_F_from_millikelvin(self):
        converter = self.degree_F.make_converter(self.millikelvin)
        self.assertAlmostEqual(converter(-459.66999999999996), 0.0)
        self.assertAlmostEqual(converter(0.0), 255372.22222222222)
        self.assertAlmostEqual(converter(32.0), 273150.0)
        self.assertAlmostEqual(converter(212.0), 373150.0)


    def test_convert_from_unit_degree_F_from_degree_C(self):
        self.assertAlmostEqual(self.degree_C.convert_from_unit(-459.66999999999996, self.degree_F), -273.14999999999998)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(0.0, self.degree_F), -17.777777777777779)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(32.0, self.degree_F), 0.0)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(212.0, self.degree_F), 100.0)

    def test_make_converter_degree_F_from_degree_C(self):
        converter = self.degree_F.make_converter(self.degree_C)
        self.assertAlmostEqual(converter(-459.66999999999996), -273.14999999999998)
        self.assertAlmostEqual(converter(0.0), -17.777777777777779)
        self.assertAlmostEqual(converter(32.0), 0.0)
        self.assertAlmostEqual(converter(212.0), 100.0)


    def test_convert_from_unit_degree_F_from_rankine_named(self):
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(-459.66999999999996, self.degree_F), 0.0)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(0.0, self.degree_F), 459.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(32.0, self.degree_F), 491.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(212.0, self.degree_F), 671.66999999999996)

    def test_make_converter_degree_F_from_rankine_named(self):
        converter = self.degree_F.make_converter(self.rankine_named)
        self.assertAlmostEqual(converter(-459.66999999999996), 0.0)
        self.assertAlmostEqual(converter(0.0), 459.66999999999996)
        self.assertAlmostEqual(converter(32.0), 491.66999999999996)
        self.assertAlmostEqual(converter(212.0), 671.66999999999996)


    def test_convert_from_unit_degree_F_from_kelvin_named(self):
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(-459.66999999999996, self.degree_F), 0.0)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(0.0, self.degree_F), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(32.0, self.degree_F), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(212.0, self.degree_F), 373.14999999999998)

    def test_make_converter_degree_F_from_kelvin_named(self):
        converter = self.degree_F.make_converter(self.kelvin_named)
        self.assertAlmostEqual(converter(-459.66999999999996), 0.0)
        self.assertAlmostEqual(converter(0.0), 255.37222222222221)
        self.assertAlmostEqual(converter(32.0), 273.14999999999998)
        self.assertAlmostEqual(converter(212.0), 373.14999999999998)


    def test_convert_from_unit_kelvin_base_from_degree_F(self):
        self.assertAlmostEqual(self.degree_F.convert_from_unit(0.0, self.kelvin_base), -459.66999999999996)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(255.37222222222221, self.kelvin_base), 0.0)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(273.14999999999998, self.kelvin_base), 32.0)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(373.14999999999998, self.kelvin_base), 212.0)

    def test_make_converter_kelvin_base_from_degree_F(self):
        converter = self.kelvin_base.make_converter(self.degree_F)
        self.assertAlmostEqual(converter(0.0), -459.66999999999996)
        self.assertAlmostEqual(converter(255.37222222222221), 0.0)
        self.assertAlmostEqual(converter(273.14999999999998), 32.0)
        self.assertAlmostEqual(converter(373.14999999999998), 212.0)


    def test_convert_from_unit_kelvin_base_from_kelvin_base(self):
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(0.0, self.kelvin_base), 0.0)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(255.37222222222221, self.kelvin_base), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(273.14999999999998, self.kelvin_base), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(373.14999999999998, self.kelvin_base), 373.14999999999998)

    def test_make_converter_kelvin_base_from_kelvin_base(self):
        converter = self.kelvin_base.make_converter(self.kelvin_base)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 255.37222222222221)
        self.assertAlmostEqual(converter(273.14999999999998), 273.14999999999998)
        self.assertAlmostEqual(converter(373.14999999999998), 373.14999999999998)


    def test_convert_from_unit_kelvin_base_from_rankine(self):
        self.assertAlmostEqual(self.rankine.convert_from_unit(0.0, self.kelvin_base), 0.0)
        self.assertAlmostEqual(self.rankine.convert_from_unit(255.37222222222221, self.kelvin_base), 459.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_unit(273.14999999999998, self.kelvin_base), 491.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_unit(373.14999999999998, self.kelvin_base), 671.66999999999996)

    def test_make_converter_kelvin_base_from_rankine(self):
        converter = self.kelvin_base.make_converter(self.rankine)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 459.66999999999996)
        self.assertAlmostEqual(converter(273.14999999999998), 491.66999999999996)
        self.assertAlmostEqual(converter(373.14999999999998), 671.66999999999996)


    def test_convert_from_unit_kelvin_base_from_kelvin(self):
        self.assertAlmostEqual(self.kelvin.convert_from_unit(0.0, self.kelvin_base), 0.0)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(255.37222222222221, self.kelvin_base), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(273.14999999999998, self.kelvin_base), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(373.14999999999998, self.kelvin_base), 373.14999999999998)

    def test_make_converter_kelvin_base_from_kelvin(self):
        converter = self.kelvin_base.make_converter(self.kelvin)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 255.37222222222221)
        self.assertAlmostEqual(converter(273.14999999999998), 273.14999999999998)
        self.assertAlmostEqual(converter(373.14999999999998), 373.14999999999998)


    def test_convert_from_unit_kelvin_base_from_millikelvin_named(self):
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(0.0, self.kelvin_base), 0.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(255.37222222222221, self.kelvin_base), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(273.14999999999998, self.kelvin_base), 273150.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(373.14999999999998, self.kelvin_base), 373150.0)

    def test_make_converter_kelvin_base_from_millikelvin_named(self):
        converter = self.kelvin_base.make_converter(self.millikelvin_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 255372.22222222222)
        self.assertAlmostEqual(converter(273.14999999999998), 273150.0)
        self.assertAlmostEqual(converter(373.14999999999998), 373150.0)


    def test_convert_from_unit_kelvin_base_from_millikelvin(self):
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(0.0, self.kelvin_base), 0.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(255.37222222222221, self.kelvin_base), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(273.14999999999998, self.kelvin_base), 273150.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(373.14999999999998, self.kelvin_base), 373150.0)

    def test_make_converter_kelvin_base_from_millikelvin(self):
        converter = self.kelvin_base.make_converter(self.millikelvin)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 255372.22222222222)
        self.assertAlmostEqual(converter(273.14999999999998), 273150.0)
        self.assertAlmostEqual(converter(373.14999999999998), 373150.0)


    def test_convert_from_unit_kelvin_base_from_degree_C(self):
        self.assertAlmostEqual(self.degree_C.convert_from_unit(0.0, self.kelvin_base), -273.14999999999998)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(255.37222222222221, self.kelvin_base), -17.777777777777779)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(273.14999999999998, self.kelvin_base), 0.0)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(373.14999999999998, self.kelvin_base), 100.0)

    def test_make_converter_kelvin_base_from_degree_C(self):
        converter = self.kelvin_base.make_converter(self.degree_C)
        self.assertAlmostEqual(converter(0.0), -273.14999999999998)
        self.assertAlmostEqual(converter(255.37222222222221), -17.777777777777779)
        self.assertAlmostEqual(converter(273.14999999999998), 0.0)
        self.assertAlmostEqual(converter(373.14999999999998), 100.0)


    def test_convert_from_unit_kelvin_base_from_rankine_named(self):
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(0.0, self.kelvin_base), 0.0)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(255.37222222222221, self.kelvin_base), 459.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(273.14999999999998, self.kelvin_base), 491.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(373.14999999999998, self.kelvin_base), 671.66999999999996)

    def test_make_converter_kelvin_base_from_rankine_named(self):
        converter = self.kelvin_base.make_converter(self.rankine_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 459.66999999999996)
        self.assertAlmostEqual(converter(273.14999999999998), 491.66999999999996)
        self.assertAlmostEqual(converter(373.14999999999998), 671.66999999999996)


    def test_convert_from_unit_kelvin_base_from_kelvin_named(self):
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(0.0, self.kelvin_base), 0.0)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(255.37222222222221, self.kelvin_base), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(273.14999999999998, self.kelvin_base), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(373.14999999999998, self.kelvin_base), 373.14999999999998)

    def test_make_converter_kelvin_base_from_kelvin_named(self):
        converter = self.kelvin_base.make_converter(self.kelvin_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 255.37222222222221)
        self.assertAlmostEqual(converter(273.14999999999998), 273.14999999999998)
        self.assertAlmostEqual(converter(373.14999999999998), 373.14999999999998)


    def test_convert_from_unit_rankine_from_degree_F(self):
        self.assertAlmostEqual(self.degree_F.convert_from_unit(0.0, self.rankine), -459.66999999999996)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(459.66999999999996, self.rankine), 0.0)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(491.66999999999996, self.rankine), 32.0)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(671.66999999999996, self.rankine), 212.0)

    def test_make_converter_rankine_from_degree_F(self):
        converter = self.rankine.make_converter(self.degree_F)
        self.assertAlmostEqual(converter(0.0), -459.66999999999996)
        self.assertAlmostEqual(converter(459.66999999999996), 0.0)
        self.assertAlmostEqual(converter(491.66999999999996), 32.0)
        self.assertAlmostEqual(converter(671.66999999999996), 212.0)


    def test_convert_from_unit_rankine_from_kelvin_base(self):
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(0.0, self.rankine), 0.0)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(459.66999999999996, self.rankine), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(491.66999999999996, self.rankine), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(671.66999999999996, self.rankine), 373.14999999999998)

    def test_make_converter_rankine_from_kelvin_base(self):
        converter = self.rankine.make_converter(self.kelvin_base)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(459.66999999999996), 255.37222222222221)
        self.assertAlmostEqual(converter(491.66999999999996), 273.14999999999998)
        self.assertAlmostEqual(converter(671.66999999999996), 373.14999999999998)


    def test_convert_from_unit_rankine_from_rankine(self):
        self.assertAlmostEqual(self.rankine.convert_from_unit(0.0, self.rankine), 0.0)
        self.assertAlmostEqual(self.rankine.convert_from_unit(459.66999999999996, self.rankine), 459.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_unit(491.66999999999996, self.rankine), 491.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_unit(671.66999999999996, self.rankine), 671.66999999999996)

    def test_make_converter_rankine_from_rankine(self):
        converter = self.rankine.make_converter(self.rankine)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(459.66999999999996), 459.66999999999996)
        self.assertAlmostEqual(converter(491.66999999999996), 491.66999999999996)
        self.assertAlmostEqual(converter(671.66999999999996), 671.66999999999996)


    def test_convert_from_unit_rankine_from_kelvin(self):
        self.assertAlmostEqual(self.kelvin.convert_from_unit(0.0, self.rankine), 0.0)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(459.66999999999996, self.rankine), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(491.66999999999996, self.rankine), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(671.66999999999996, self.rankine), 373.14999999999998)

    def test_make_converter_rankine_from_kelvin(self):
        converter = self.rankine.make_converter(self.kelvin)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(459.66999999999996), 255.37222222222221)
        self.assertAlmostEqual(converter(491.66999999999996), 273.14999999999998)
        self.assertAlmostEqual(converter(671.66999999999996), 373.14999999999998)


    def test_convert_from_unit_rankine_from_millikelvin_named(self):
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(0.0, self.rankine), 0.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(459.66999999999996, self.rankine), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(491.66999999999996, self.rankine), 273150.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(671.66999999999996, self.rankine), 373150.0)

    def test_make_converter_rankine_from_millikelvin_named(self):
        converter = self.rankine.make_converter(self.millikelvin_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(459.66999999999996), 255372.22222222222)
        self.assertAlmostEqual(converter(491.66999999999996), 273150.0)
        self.assertAlmostEqual(converter(671.66999999999996), 373150.0)


    def test_convert_from_unit_rankine_from_millikelvin(self):
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(0.0, self.rankine), 0.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(459.66999999999996, self.rankine), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(491.66999999999996, self.rankine), 273150.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(671.66999999999996, self.rankine), 373150.0)

    def test_make_converter_rankine_from_millikelvin(self):
        converter = self.rankine.make_converter(self.millikelvin)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(459.66999999999996), 255372.22222222222)
        self.assertAlmostEqual(converter(491.66999999999996), 273150.0)
        self.assertAlmostEqual(converter(671.66999999999996), 373150.0)


    def test_convert_from_unit_rankine_from_degree_C(self):
        self.assertAlmostEqual(self.degree_C.convert_from_unit(0.0, self.rankine), -273.14999999999998)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(459.66999999999996, self.rankine), -17.777777777777779)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(491.66999999999996, self.rankine), 0.0)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(671.66999999999996, self.rankine), 100.0)

    def test_make_converter_rankine_from_degree_C(self):
        converter = self.rankine.make_converter(self.degree_C)
        self.assertAlmostEqual(converter(0.0), -273.14999999999998)
        self.assertAlmostEqual(converter(459.66999999999996), -17.777777777777779)
        self.assertAlmostEqual(converter(491.66999999999996), 0.0)
        self.assertAlmostEqual(converter(671.66999999999996), 100.0)


    def test_convert_from_unit_rankine_from_rankine_named(self):
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(0.0, self.rankine), 0.0)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(459.66999999999996, self.rankine), 459.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(491.66999999999996, self.rankine), 491.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(671.66999999999996, self.rankine), 671.66999999999996)

    def test_make_converter_rankine_from_rankine_named(self):
        converter = self.rankine.make_converter(self.rankine_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(459.66999999999996), 459.66999999999996)
        self.assertAlmostEqual(converter(491.66999999999996), 491.66999999999996)
        self.assertAlmostEqual(converter(671.66999999999996), 671.66999999999996)


    def test_convert_from_unit_rankine_from_kelvin_named(self):
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(0.0, self.rankine), 0.0)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(459.66999999999996, self.rankine), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(491.66999999999996, self.rankine), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(671.66999999999996, self.rankine), 373.14999999999998)

    def test_make_converter_rankine_from_kelvin_named(self):
        converter = self.rankine.make_converter(self.kelvin_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(459.66999999999996), 255.37222222222221)
        self.assertAlmostEqual(converter(491.66999999999996), 273.14999999999998)
        self.assertAlmostEqual(converter(671.66999999999996), 373.14999999999998)


    def test_convert_from_unit_kelvin_from_degree_F(self):
        self.assertAlmostEqual(self.degree_F.convert_from_unit(0.0, self.kelvin), -459.66999999999996)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(255.37222222222221, self.kelvin), 0.0)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(273.14999999999998, self.kelvin), 32.0)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(373.14999999999998, self.kelvin), 212.0)

    def test_make_converter_kelvin_from_degree_F(self):
        converter = self.kelvin.make_converter(self.degree_F)
        self.assertAlmostEqual(converter(0.0), -459.66999999999996)
        self.assertAlmostEqual(converter(255.37222222222221), 0.0)
        self.assertAlmostEqual(converter(273.14999999999998), 32.0)
        self.assertAlmostEqual(converter(373.14999999999998), 212.0)


    def test_convert_from_unit_kelvin_from_kelvin_base(self):
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(0.0, self.kelvin), 0.0)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(255.37222222222221, self.kelvin), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(273.14999999999998, self.kelvin), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(373.14999999999998, self.kelvin), 373.14999999999998)

    def test_make_converter_kelvin_from_kelvin_base(self):
        converter = self.kelvin.make_converter(self.kelvin_base)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 255.37222222222221)
        self.assertAlmostEqual(converter(273.14999999999998), 273.14999999999998)
        self.assertAlmostEqual(converter(373.14999999999998), 373.14999999999998)


    def test_convert_from_unit_kelvin_from_rankine(self):
        self.assertAlmostEqual(self.rankine.convert_from_unit(0.0, self.kelvin), 0.0)
        self.assertAlmostEqual(self.rankine.convert_from_unit(255.37222222222221, self.kelvin), 459.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_unit(273.14999999999998, self.kelvin), 491.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_unit(373.14999999999998, self.kelvin), 671.66999999999996)

    def test_make_converter_kelvin_from_rankine(self):
        converter = self.kelvin.make_converter(self.rankine)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 459.66999999999996)
        self.assertAlmostEqual(converter(273.14999999999998), 491.66999999999996)
        self.assertAlmostEqual(converter(373.14999999999998), 671.66999999999996)


    def test_convert_from_unit_kelvin_from_kelvin(self):
        self.assertAlmostEqual(self.kelvin.convert_from_unit(0.0, self.kelvin), 0.0)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(255.37222222222221, self.kelvin), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(273.14999999999998, self.kelvin), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(373.14999999999998, self.kelvin), 373.14999999999998)

    def test_make_converter_kelvin_from_kelvin(self):
        converter = self.kelvin.make_converter(self.kelvin)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 255.37222222222221)
        self.assertAlmostEqual(converter(273.14999999999998), 273.14999999999998)
        self.assertAlmostEqual(converter(373.14999999999998), 373.14999999999998)


    def test_convert_from_unit_kelvin_from_millikelvin_named(self):
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(0.0, self.kelvin), 0.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(255.37222222222221, self.kelvin), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(273.14999999999998, self.kelvin), 273150.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(373.14999999999998, self.kelvin), 373150.0)

    def test_make_converter_kelvin_from_millikelvin_named(self):
        converter = self.kelvin.make_converter(self.millikelvin_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 255372.22222222222)
        self.assertAlmostEqual(converter(273.14999999999998), 273150.0)
        self.assertAlmostEqual(converter(373.14999999999998), 373150.0)


    def test_convert_from_unit_kelvin_from_millikelvin(self):
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(0.0, self.kelvin), 0.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(255.37222222222221, self.kelvin), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(273.14999999999998, self.kelvin), 273150.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(373.14999999999998, self.kelvin), 373150.0)

    def test_make_converter_kelvin_from_millikelvin(self):
        converter = self.kelvin.make_converter(self.millikelvin)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 255372.22222222222)
        self.assertAlmostEqual(converter(273.14999999999998), 273150.0)
        self.assertAlmostEqual(converter(373.14999999999998), 373150.0)


    def test_convert_from_unit_kelvin_from_degree_C(self):
        self.assertAlmostEqual(self.degree_C.convert_from_unit(0.0, self.kelvin), -273.14999999999998)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(255.37222222222221, self.kelvin), -17.777777777777779)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(273.14999999999998, self.kelvin), 0.0)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(373.14999999999998, self.kelvin), 100.0)

    def test_make_converter_kelvin_from_degree_C(self):
        converter = self.kelvin.make_converter(self.degree_C)
        self.assertAlmostEqual(converter(0.0), -273.14999999999998)
        self.assertAlmostEqual(converter(255.37222222222221), -17.777777777777779)
        self.assertAlmostEqual(converter(273.14999999999998), 0.0)
        self.assertAlmostEqual(converter(373.14999999999998), 100.0)


    def test_convert_from_unit_kelvin_from_rankine_named(self):
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(0.0, self.kelvin), 0.0)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(255.37222222222221, self.kelvin), 459.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(273.14999999999998, self.kelvin), 491.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(373.14999999999998, self.kelvin), 671.66999999999996)

    def test_make_converter_kelvin_from_rankine_named(self):
        converter = self.kelvin.make_converter(self.rankine_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 459.66999999999996)
        self.assertAlmostEqual(converter(273.14999999999998), 491.66999999999996)
        self.assertAlmostEqual(converter(373.14999999999998), 671.66999999999996)


    def test_convert_from_unit_kelvin_from_kelvin_named(self):
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(0.0, self.kelvin), 0.0)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(255.37222222222221, self.kelvin), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(273.14999999999998, self.kelvin), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(373.14999999999998, self.kelvin), 373.14999999999998)

    def test_make_converter_kelvin_from_kelvin_named(self):
        converter = self.kelvin.make_converter(self.kelvin_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 255.37222222222221)
        self.assertAlmostEqual(converter(273.14999999999998), 273.14999999999998)
        self.assertAlmostEqual(converter(373.14999999999998), 373.14999999999998)


    def test_convert_from_unit_millikelvin_named_from_degree_F(self):
        self.assertAlmostEqual(self.degree_F.convert_from_unit(0.0, self.millikelvin_named), -459.66999999999996)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(255372.22222222222, self.millikelvin_named), 0.0)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(273150.0, self.millikelvin_named), 32.0)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(373150.0, self.millikelvin_named), 212.0)

    def test_make_converter_millikelvin_named_from_degree_F(self):
        converter = self.millikelvin_named.make_converter(self.degree_F)
        self.assertAlmostEqual(converter(0.0), -459.66999999999996)
        self.assertAlmostEqual(converter(255372.22222222222), 0.0)
        self.assertAlmostEqual(converter(273150.0), 32.0)
        self.assertAlmostEqual(converter(373150.0), 212.0)


    def test_convert_from_unit_millikelvin_named_from_kelvin_base(self):
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(0.0, self.millikelvin_named), 0.0)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(255372.22222222222, self.millikelvin_named), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(273150.0, self.millikelvin_named), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(373150.0, self.millikelvin_named), 373.14999999999998)

    def test_make_converter_millikelvin_named_from_kelvin_base(self):
        converter = self.millikelvin_named.make_converter(self.kelvin_base)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255372.22222222222), 255.37222222222221)
        self.assertAlmostEqual(converter(273150.0), 273.14999999999998)
        self.assertAlmostEqual(converter(373150.0), 373.14999999999998)


    def test_convert_from_unit_millikelvin_named_from_rankine(self):
        self.assertAlmostEqual(self.rankine.convert_from_unit(0.0, self.millikelvin_named), 0.0)
        self.assertAlmostEqual(self.rankine.convert_from_unit(255372.22222222222, self.millikelvin_named), 459.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_unit(273150.0, self.millikelvin_named), 491.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_unit(373150.0, self.millikelvin_named), 671.66999999999996)

    def test_make_converter_millikelvin_named_from_rankine(self):
        converter = self.millikelvin_named.make_converter(self.rankine)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255372.22222222222), 459.66999999999996)
        self.assertAlmostEqual(converter(273150.0), 491.66999999999996)
        self.assertAlmostEqual(converter(373150.0), 671.66999999999996)


    def test_convert_from_unit_millikelvin_named_from_kelvin(self):
        self.assertAlmostEqual(self.kelvin.convert_from_unit(0.0, self.millikelvin_named), 0.0)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(255372.22222222222, self.millikelvin_named), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(273150.0, self.millikelvin_named), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(373150.0, self.millikelvin_named), 373.14999999999998)

    def test_make_converter_millikelvin_named_from_kelvin(self):
        converter = self.millikelvin_named.make_converter(self.kelvin)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255372.22222222222), 255.37222222222221)
        self.assertAlmostEqual(converter(273150.0), 273.14999999999998)
        self.assertAlmostEqual(converter(373150.0), 373.14999999999998)


    def test_convert_from_unit_millikelvin_named_from_millikelvin_named(self):
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(0.0, self.millikelvin_named), 0.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(255372.22222222222, self.millikelvin_named), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(273150.0, self.millikelvin_named), 273150.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(373150.0, self.millikelvin_named), 373150.0)

    def test_make_converter_millikelvin_named_from_millikelvin_named(self):
        converter = self.millikelvin_named.make_converter(self.millikelvin_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255372.22222222222), 255372.22222222222)
        self.assertAlmostEqual(converter(273150.0), 273150.0)
        self.assertAlmostEqual(converter(373150.0), 373150.0)


    def test_convert_from_unit_millikelvin_named_from_millikelvin(self):
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(0.0, self.millikelvin_named), 0.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(255372.22222222222, self.millikelvin_named), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(273150.0, self.millikelvin_named), 273150.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(373150.0, self.millikelvin_named), 373150.0)

    def test_make_converter_millikelvin_named_from_millikelvin(self):
        converter = self.millikelvin_named.make_converter(self.millikelvin)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255372.22222222222), 255372.22222222222)
        self.assertAlmostEqual(converter(273150.0), 273150.0)
        self.assertAlmostEqual(converter(373150.0), 373150.0)


    def test_convert_from_unit_millikelvin_named_from_degree_C(self):
        self.assertAlmostEqual(self.degree_C.convert_from_unit(0.0, self.millikelvin_named), -273.14999999999998)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(255372.22222222222, self.millikelvin_named), -17.777777777777779)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(273150.0, self.millikelvin_named), 0.0)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(373150.0, self.millikelvin_named), 100.0)

    def test_make_converter_millikelvin_named_from_degree_C(self):
        converter = self.millikelvin_named.make_converter(self.degree_C)
        self.assertAlmostEqual(converter(0.0), -273.14999999999998)
        self.assertAlmostEqual(converter(255372.22222222222), -17.777777777777779)
        self.assertAlmostEqual(converter(273150.0), 0.0)
        self.assertAlmostEqual(converter(373150.0), 100.0)


    def test_convert_from_unit_millikelvin_named_from_rankine_named(self):
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(0.0, self.millikelvin_named), 0.0)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(255372.22222222222, self.millikelvin_named), 459.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(273150.0, self.millikelvin_named), 491.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(373150.0, self.millikelvin_named), 671.66999999999996)

    def test_make_converter_millikelvin_named_from_rankine_named(self):
        converter = self.millikelvin_named.make_converter(self.rankine_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255372.22222222222), 459.66999999999996)
        self.assertAlmostEqual(converter(273150.0), 491.66999999999996)
        self.assertAlmostEqual(converter(373150.0), 671.66999999999996)


    def test_convert_from_unit_millikelvin_named_from_kelvin_named(self):
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(0.0, self.millikelvin_named), 0.0)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(255372.22222222222, self.millikelvin_named), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(273150.0, self.millikelvin_named), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(373150.0, self.millikelvin_named), 373.14999999999998)

    def test_make_converter_millikelvin_named_from_kelvin_named(self):
        converter = self.millikelvin_named.make_converter(self.kelvin_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255372.22222222222), 255.37222222222221)
        self.assertAlmostEqual(converter(273150.0), 273.14999999999998)
        self.assertAlmostEqual(converter(373150.0), 373.14999999999998)


    def test_convert_from_unit_millikelvin_from_degree_F(self):
        self.assertAlmostEqual(self.degree_F.convert_from_unit(0.0, self.millikelvin), -459.66999999999996)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(255372.22222222222, self.millikelvin), 0.0)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(273150.0, self.millikelvin), 32.0)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(373150.0, self.millikelvin), 212.0)

    def test_make_converter_millikelvin_from_degree_F(self):
        converter = self.millikelvin.make_converter(self.degree_F)
        self.assertAlmostEqual(converter(0.0), -459.66999999999996)
        self.assertAlmostEqual(converter(255372.22222222222), 0.0)
        self.assertAlmostEqual(converter(273150.0), 32.0)
        self.assertAlmostEqual(converter(373150.0), 212.0)


    def test_convert_from_unit_millikelvin_from_kelvin_base(self):
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(0.0, self.millikelvin), 0.0)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(255372.22222222222, self.millikelvin), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(273150.0, self.millikelvin), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(373150.0, self.millikelvin), 373.14999999999998)

    def test_make_converter_millikelvin_from_kelvin_base(self):
        converter = self.millikelvin.make_converter(self.kelvin_base)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255372.22222222222), 255.37222222222221)
        self.assertAlmostEqual(converter(273150.0), 273.14999999999998)
        self.assertAlmostEqual(converter(373150.0), 373.14999999999998)


    def test_convert_from_unit_millikelvin_from_rankine(self):
        self.assertAlmostEqual(self.rankine.convert_from_unit(0.0, self.millikelvin), 0.0)
        self.assertAlmostEqual(self.rankine.convert_from_unit(255372.22222222222, self.millikelvin), 459.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_unit(273150.0, self.millikelvin), 491.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_unit(373150.0, self.millikelvin), 671.66999999999996)

    def test_make_converter_millikelvin_from_rankine(self):
        converter = self.millikelvin.make_converter(self.rankine)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255372.22222222222), 459.66999999999996)
        self.assertAlmostEqual(converter(273150.0), 491.66999999999996)
        self.assertAlmostEqual(converter(373150.0), 671.66999999999996)


    def test_convert_from_unit_millikelvin_from_kelvin(self):
        self.assertAlmostEqual(self.kelvin.convert_from_unit(0.0, self.millikelvin), 0.0)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(255372.22222222222, self.millikelvin), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(273150.0, self.millikelvin), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(373150.0, self.millikelvin), 373.14999999999998)

    def test_make_converter_millikelvin_from_kelvin(self):
        converter = self.millikelvin.make_converter(self.kelvin)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255372.22222222222), 255.37222222222221)
        self.assertAlmostEqual(converter(273150.0), 273.14999999999998)
        self.assertAlmostEqual(converter(373150.0), 373.14999999999998)


    def test_convert_from_unit_millikelvin_from_millikelvin_named(self):
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(0.0, self.millikelvin), 0.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(255372.22222222222, self.millikelvin), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(273150.0, self.millikelvin), 273150.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(373150.0, self.millikelvin), 373150.0)

    def test_make_converter_millikelvin_from_millikelvin_named(self):
        converter = self.millikelvin.make_converter(self.millikelvin_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255372.22222222222), 255372.22222222222)
        self.assertAlmostEqual(converter(273150.0), 273150.0)
        self.assertAlmostEqual(converter(373150.0), 373150.0)


    def test_convert_from_unit_millikelvin_from_millikelvin(self):
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(0.0, self.millikelvin), 0.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(255372.22222222222, self.millikelvin), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(273150.0, self.millikelvin), 273150.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(373150.0, self.millikelvin), 373150.0)

    def test_make_converter_millikelvin_from_millikelvin(self):
        converter = self.millikelvin.make_converter(self.millikelvin)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255372.22222222222), 255372.22222222222)
        self.assertAlmostEqual(converter(273150.0), 273150.0)
        self.assertAlmostEqual(converter(373150.0), 373150.0)


    def test_convert_from_unit_millikelvin_from_degree_C(self):
        self.assertAlmostEqual(self.degree_C.convert_from_unit(0.0, self.millikelvin), -273.14999999999998)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(255372.22222222222, self.millikelvin), -17.777777777777779)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(273150.0, self.millikelvin), 0.0)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(373150.0, self.millikelvin), 100.0)

    def test_make_converter_millikelvin_from_degree_C(self):
        converter = self.millikelvin.make_converter(self.degree_C)
        self.assertAlmostEqual(converter(0.0), -273.14999999999998)
        self.assertAlmostEqual(converter(255372.22222222222), -17.777777777777779)
        self.assertAlmostEqual(converter(273150.0), 0.0)
        self.assertAlmostEqual(converter(373150.0), 100.0)


    def test_convert_from_unit_millikelvin_from_rankine_named(self):
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(0.0, self.millikelvin), 0.0)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(255372.22222222222, self.millikelvin), 459.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(273150.0, self.millikelvin), 491.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(373150.0, self.millikelvin), 671.66999999999996)

    def test_make_converter_millikelvin_from_rankine_named(self):
        converter = self.millikelvin.make_converter(self.rankine_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255372.22222222222), 459.66999999999996)
        self.assertAlmostEqual(converter(273150.0), 491.66999999999996)
        self.assertAlmostEqual(converter(373150.0), 671.66999999999996)


    def test_convert_from_unit_millikelvin_from_kelvin_named(self):
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(0.0, self.millikelvin), 0.0)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(255372.22222222222, self.millikelvin), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(273150.0, self.millikelvin), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(373150.0, self.millikelvin), 373.14999999999998)

    def test_make_converter_millikelvin_from_kelvin_named(self):
        converter = self.millikelvin.make_converter(self.kelvin_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255372.22222222222), 255.37222222222221)
        self.assertAlmostEqual(converter(273150.0), 273.14999999999998)
        self.assertAlmostEqual(converter(373150.0), 373.14999999999998)


    def test_convert_from_unit_degree_C_from_degree_F(self):
        self.assertAlmostEqual(self.degree_F.convert_from_unit(-273.14999999999998, self.degree_C), -459.66999999999996)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(-17.777777777777779, self.degree_C), 0.0)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(0.0, self.degree_C), 32.0)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(100.0, self.degree_C), 212.0)

    def test_make_converter_degree_C_from_degree_F(self):
        converter = self.degree_C.make_converter(self.degree_F)
        self.assertAlmostEqual(converter(-273.14999999999998), -459.66999999999996)
        self.assertAlmostEqual(converter(-17.777777777777779), 0.0)
        self.assertAlmostEqual(converter(0.0), 32.0)
        self.assertAlmostEqual(converter(100.0), 212.0)


    def test_convert_from_unit_degree_C_from_kelvin_base(self):
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(-273.14999999999998, self.degree_C), 0.0)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(-17.777777777777779, self.degree_C), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(0.0, self.degree_C), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(100.0, self.degree_C), 373.14999999999998)

    def test_make_converter_degree_C_from_kelvin_base(self):
        converter = self.degree_C.make_converter(self.kelvin_base)
        self.assertAlmostEqual(converter(-273.14999999999998), 0.0)
        self.assertAlmostEqual(converter(-17.777777777777779), 255.37222222222221)
        self.assertAlmostEqual(converter(0.0), 273.14999999999998)
        self.assertAlmostEqual(converter(100.0), 373.14999999999998)


    def test_convert_from_unit_degree_C_from_rankine(self):
        self.assertAlmostEqual(self.rankine.convert_from_unit(-273.14999999999998, self.degree_C), 0.0)
        self.assertAlmostEqual(self.rankine.convert_from_unit(-17.777777777777779, self.degree_C), 459.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_unit(0.0, self.degree_C), 491.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_unit(100.0, self.degree_C), 671.66999999999996)

    def test_make_converter_degree_C_from_rankine(self):
        converter = self.degree_C.make_converter(self.rankine)
        self.assertAlmostEqual(converter(-273.14999999999998), 0.0)
        self.assertAlmostEqual(converter(-17.777777777777779), 459.66999999999996)
        self.assertAlmostEqual(converter(0.0), 491.66999999999996)
        self.assertAlmostEqual(converter(100.0), 671.66999999999996)


    def test_convert_from_unit_degree_C_from_kelvin(self):
        self.assertAlmostEqual(self.kelvin.convert_from_unit(-273.14999999999998, self.degree_C), 0.0)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(-17.777777777777779, self.degree_C), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(0.0, self.degree_C), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(100.0, self.degree_C), 373.14999999999998)

    def test_make_converter_degree_C_from_kelvin(self):
        converter = self.degree_C.make_converter(self.kelvin)
        self.assertAlmostEqual(converter(-273.14999999999998), 0.0)
        self.assertAlmostEqual(converter(-17.777777777777779), 255.37222222222221)
        self.assertAlmostEqual(converter(0.0), 273.14999999999998)
        self.assertAlmostEqual(converter(100.0), 373.14999999999998)


    def test_convert_from_unit_degree_C_from_millikelvin_named(self):
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(-273.14999999999998, self.degree_C), 0.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(-17.777777777777779, self.degree_C), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(0.0, self.degree_C), 273150.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(100.0, self.degree_C), 373150.0)

    def test_make_converter_degree_C_from_millikelvin_named(self):
        converter = self.degree_C.make_converter(self.millikelvin_named)
        self.assertAlmostEqual(converter(-273.14999999999998), 0.0)
        self.assertAlmostEqual(converter(-17.777777777777779), 255372.22222222222)
        self.assertAlmostEqual(converter(0.0), 273150.0)
        self.assertAlmostEqual(converter(100.0), 373150.0)


    def test_convert_from_unit_degree_C_from_millikelvin(self):
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(-273.14999999999998, self.degree_C), 0.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(-17.777777777777779, self.degree_C), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(0.0, self.degree_C), 273150.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(100.0, self.degree_C), 373150.0)

    def test_make_converter_degree_C_from_millikelvin(self):
        converter = self.degree_C.make_converter(self.millikelvin)
        self.assertAlmostEqual(converter(-273.14999999999998), 0.0)
        self.assertAlmostEqual(converter(-17.777777777777779), 255372.22222222222)
        self.assertAlmostEqual(converter(0.0), 273150.0)
        self.assertAlmostEqual(converter(100.0), 373150.0)


    def test_convert_from_unit_degree_C_from_degree_C(self):
        self.assertAlmostEqual(self.degree_C.convert_from_unit(-273.14999999999998, self.degree_C), -273.14999999999998)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(-17.777777777777779, self.degree_C), -17.777777777777779)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(0.0, self.degree_C), 0.0)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(100.0, self.degree_C), 100.0)

    def test_make_converter_degree_C_from_degree_C(self):
        converter = self.degree_C.make_converter(self.degree_C)
        self.assertAlmostEqual(converter(-273.14999999999998), -273.14999999999998)
        self.assertAlmostEqual(converter(-17.777777777777779), -17.777777777777779)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(100.0), 100.0)


    def test_convert_from_unit_degree_C_from_rankine_named(self):
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(-273.14999999999998, self.degree_C), 0.0)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(-17.777777777777779, self.degree_C), 459.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(0.0, self.degree_C), 491.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(100.0, self.degree_C), 671.66999999999996)

    def test_make_converter_degree_C_from_rankine_named(self):
        converter = self.degree_C.make_converter(self.rankine_named)
        self.assertAlmostEqual(converter(-273.14999999999998), 0.0)
        self.assertAlmostEqual(converter(-17.777777777777779), 459.66999999999996)
        self.assertAlmostEqual(converter(0.0), 491.66999999999996)
        self.assertAlmostEqual(converter(100.0), 671.66999999999996)


    def test_convert_from_unit_degree_C_from_kelvin_named(self):
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(-273.14999999999998, self.degree_C), 0.0)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(-17.777777777777779, self.degree_C), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(0.0, self.degree_C), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(100.0, self.degree_C), 373.14999999999998)

    def test_make_converter_degree_C_from_kelvin_named(self):
        converter = self.degree_C.make_converter(self.kelvin_named)
        self.assertAlmostEqual(converter(-273.14999999999998), 0.0)
        self.assertAlmostEqual(converter(-17.777777777777779), 255.37222222222221)
        self.assertAlmostEqual(converter(0.0), 273.14999999999998)
        self.assertAlmostEqual(converter(100.0), 373.14999999999998)


    def test_convert_from_unit_rankine_named_from_degree_F(self):
        self.assertAlmostEqual(self.degree_F.convert_from_unit(0.0, self.rankine_named), -459.66999999999996)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(459.66999999999996, self.rankine_named), 0.0)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(491.66999999999996, self.rankine_named), 32.0)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(671.66999999999996, self.rankine_named), 212.0)

    def test_make_converter_rankine_named_from_degree_F(self):
        converter = self.rankine_named.make_converter(self.degree_F)
        self.assertAlmostEqual(converter(0.0), -459.66999999999996)
        self.assertAlmostEqual(converter(459.66999999999996), 0.0)
        self.assertAlmostEqual(converter(491.66999999999996), 32.0)
        self.assertAlmostEqual(converter(671.66999999999996), 212.0)


    def test_convert_from_unit_rankine_named_from_kelvin_base(self):
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(0.0, self.rankine_named), 0.0)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(459.66999999999996, self.rankine_named), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(491.66999999999996, self.rankine_named), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(671.66999999999996, self.rankine_named), 373.14999999999998)

    def test_make_converter_rankine_named_from_kelvin_base(self):
        converter = self.rankine_named.make_converter(self.kelvin_base)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(459.66999999999996), 255.37222222222221)
        self.assertAlmostEqual(converter(491.66999999999996), 273.14999999999998)
        self.assertAlmostEqual(converter(671.66999999999996), 373.14999999999998)


    def test_convert_from_unit_rankine_named_from_rankine(self):
        self.assertAlmostEqual(self.rankine.convert_from_unit(0.0, self.rankine_named), 0.0)
        self.assertAlmostEqual(self.rankine.convert_from_unit(459.66999999999996, self.rankine_named), 459.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_unit(491.66999999999996, self.rankine_named), 491.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_unit(671.66999999999996, self.rankine_named), 671.66999999999996)

    def test_make_converter_rankine_named_from_rankine(self):
        converter = self.rankine_named.make_converter(self.rankine)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(459.66999999999996), 459.66999999999996)
        self.assertAlmostEqual(converter(491.66999999999996), 491.66999999999996)
        self.assertAlmostEqual(converter(671.66999999999996), 671.66999999999996)


    def test_convert_from_unit_rankine_named_from_kelvin(self):
        self.assertAlmostEqual(self.kelvin.convert_from_unit(0.0, self.rankine_named), 0.0)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(459.66999999999996, self.rankine_named), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(491.66999999999996, self.rankine_named), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(671.66999999999996, self.rankine_named), 373.14999999999998)

    def test_make_converter_rankine_named_from_kelvin(self):
        converter = self.rankine_named.make_converter(self.kelvin)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(459.66999999999996), 255.37222222222221)
        self.assertAlmostEqual(converter(491.66999999999996), 273.14999999999998)
        self.assertAlmostEqual(converter(671.66999999999996), 373.14999999999998)


    def test_convert_from_unit_rankine_named_from_millikelvin_named(self):
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(0.0, self.rankine_named), 0.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(459.66999999999996, self.rankine_named), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(491.66999999999996, self.rankine_named), 273150.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(671.66999999999996, self.rankine_named), 373150.0)

    def test_make_converter_rankine_named_from_millikelvin_named(self):
        converter = self.rankine_named.make_converter(self.millikelvin_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(459.66999999999996), 255372.22222222222)
        self.assertAlmostEqual(converter(491.66999999999996), 273150.0)
        self.assertAlmostEqual(converter(671.66999999999996), 373150.0)


    def test_convert_from_unit_rankine_named_from_millikelvin(self):
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(0.0, self.rankine_named), 0.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(459.66999999999996, self.rankine_named), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(491.66999999999996, self.rankine_named), 273150.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(671.66999999999996, self.rankine_named), 373150.0)

    def test_make_converter_rankine_named_from_millikelvin(self):
        converter = self.rankine_named.make_converter(self.millikelvin)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(459.66999999999996), 255372.22222222222)
        self.assertAlmostEqual(converter(491.66999999999996), 273150.0)
        self.assertAlmostEqual(converter(671.66999999999996), 373150.0)


    def test_convert_from_unit_rankine_named_from_degree_C(self):
        self.assertAlmostEqual(self.degree_C.convert_from_unit(0.0, self.rankine_named), -273.14999999999998)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(459.66999999999996, self.rankine_named), -17.777777777777779)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(491.66999999999996, self.rankine_named), 0.0)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(671.66999999999996, self.rankine_named), 100.0)

    def test_make_converter_rankine_named_from_degree_C(self):
        converter = self.rankine_named.make_converter(self.degree_C)
        self.assertAlmostEqual(converter(0.0), -273.14999999999998)
        self.assertAlmostEqual(converter(459.66999999999996), -17.777777777777779)
        self.assertAlmostEqual(converter(491.66999999999996), 0.0)
        self.assertAlmostEqual(converter(671.66999999999996), 100.0)


    def test_convert_from_unit_rankine_named_from_rankine_named(self):
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(0.0, self.rankine_named), 0.0)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(459.66999999999996, self.rankine_named), 459.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(491.66999999999996, self.rankine_named), 491.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(671.66999999999996, self.rankine_named), 671.66999999999996)

    def test_make_converter_rankine_named_from_rankine_named(self):
        converter = self.rankine_named.make_converter(self.rankine_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(459.66999999999996), 459.66999999999996)
        self.assertAlmostEqual(converter(491.66999999999996), 491.66999999999996)
        self.assertAlmostEqual(converter(671.66999999999996), 671.66999999999996)


    def test_convert_from_unit_rankine_named_from_kelvin_named(self):
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(0.0, self.rankine_named), 0.0)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(459.66999999999996, self.rankine_named), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(491.66999999999996, self.rankine_named), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(671.66999999999996, self.rankine_named), 373.14999999999998)

    def test_make_converter_rankine_named_from_kelvin_named(self):
        converter = self.rankine_named.make_converter(self.kelvin_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(459.66999999999996), 255.37222222222221)
        self.assertAlmostEqual(converter(491.66999999999996), 273.14999999999998)
        self.assertAlmostEqual(converter(671.66999999999996), 373.14999999999998)


    def test_convert_from_unit_kelvin_named_from_degree_F(self):
        self.assertAlmostEqual(self.degree_F.convert_from_unit(0.0, self.kelvin_named), -459.66999999999996)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(255.37222222222221, self.kelvin_named), 0.0)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(273.14999999999998, self.kelvin_named), 32.0)
        self.assertAlmostEqual(self.degree_F.convert_from_unit(373.14999999999998, self.kelvin_named), 212.0)

    def test_make_converter_kelvin_named_from_degree_F(self):
        converter = self.kelvin_named.make_converter(self.degree_F)
        self.assertAlmostEqual(converter(0.0), -459.66999999999996)
        self.assertAlmostEqual(converter(255.37222222222221), 0.0)
        self.assertAlmostEqual(converter(273.14999999999998), 32.0)
        self.assertAlmostEqual(converter(373.14999999999998), 212.0)


    def test_convert_from_unit_kelvin_named_from_kelvin_base(self):
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(0.0, self.kelvin_named), 0.0)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(255.37222222222221, self.kelvin_named), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(273.14999999999998, self.kelvin_named), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_base.convert_from_unit(373.14999999999998, self.kelvin_named), 373.14999999999998)

    def test_make_converter_kelvin_named_from_kelvin_base(self):
        converter = self.kelvin_named.make_converter(self.kelvin_base)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 255.37222222222221)
        self.assertAlmostEqual(converter(273.14999999999998), 273.14999999999998)
        self.assertAlmostEqual(converter(373.14999999999998), 373.14999999999998)


    def test_convert_from_unit_kelvin_named_from_rankine(self):
        self.assertAlmostEqual(self.rankine.convert_from_unit(0.0, self.kelvin_named), 0.0)
        self.assertAlmostEqual(self.rankine.convert_from_unit(255.37222222222221, self.kelvin_named), 459.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_unit(273.14999999999998, self.kelvin_named), 491.66999999999996)
        self.assertAlmostEqual(self.rankine.convert_from_unit(373.14999999999998, self.kelvin_named), 671.66999999999996)

    def test_make_converter_kelvin_named_from_rankine(self):
        converter = self.kelvin_named.make_converter(self.rankine)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 459.66999999999996)
        self.assertAlmostEqual(converter(273.14999999999998), 491.66999999999996)
        self.assertAlmostEqual(converter(373.14999999999998), 671.66999999999996)


    def test_convert_from_unit_kelvin_named_from_kelvin(self):
        self.assertAlmostEqual(self.kelvin.convert_from_unit(0.0, self.kelvin_named), 0.0)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(255.37222222222221, self.kelvin_named), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(273.14999999999998, self.kelvin_named), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin.convert_from_unit(373.14999999999998, self.kelvin_named), 373.14999999999998)

    def test_make_converter_kelvin_named_from_kelvin(self):
        converter = self.kelvin_named.make_converter(self.kelvin)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 255.37222222222221)
        self.assertAlmostEqual(converter(273.14999999999998), 273.14999999999998)
        self.assertAlmostEqual(converter(373.14999999999998), 373.14999999999998)


    def test_convert_from_unit_kelvin_named_from_millikelvin_named(self):
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(0.0, self.kelvin_named), 0.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(255.37222222222221, self.kelvin_named), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(273.14999999999998, self.kelvin_named), 273150.0)
        self.assertAlmostEqual(self.millikelvin_named.convert_from_unit(373.14999999999998, self.kelvin_named), 373150.0)

    def test_make_converter_kelvin_named_from_millikelvin_named(self):
        converter = self.kelvin_named.make_converter(self.millikelvin_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 255372.22222222222)
        self.assertAlmostEqual(converter(273.14999999999998), 273150.0)
        self.assertAlmostEqual(converter(373.14999999999998), 373150.0)


    def test_convert_from_unit_kelvin_named_from_millikelvin(self):
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(0.0, self.kelvin_named), 0.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(255.37222222222221, self.kelvin_named), 255372.22222222222)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(273.14999999999998, self.kelvin_named), 273150.0)
        self.assertAlmostEqual(self.millikelvin.convert_from_unit(373.14999999999998, self.kelvin_named), 373150.0)

    def test_make_converter_kelvin_named_from_millikelvin(self):
        converter = self.kelvin_named.make_converter(self.millikelvin)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 255372.22222222222)
        self.assertAlmostEqual(converter(273.14999999999998), 273150.0)
        self.assertAlmostEqual(converter(373.14999999999998), 373150.0)


    def test_convert_from_unit_kelvin_named_from_degree_C(self):
        self.assertAlmostEqual(self.degree_C.convert_from_unit(0.0, self.kelvin_named), -273.14999999999998)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(255.37222222222221, self.kelvin_named), -17.777777777777779)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(273.14999999999998, self.kelvin_named), 0.0)
        self.assertAlmostEqual(self.degree_C.convert_from_unit(373.14999999999998, self.kelvin_named), 100.0)

    def test_make_converter_kelvin_named_from_degree_C(self):
        converter = self.kelvin_named.make_converter(self.degree_C)
        self.assertAlmostEqual(converter(0.0), -273.14999999999998)
        self.assertAlmostEqual(converter(255.37222222222221), -17.777777777777779)
        self.assertAlmostEqual(converter(273.14999999999998), 0.0)
        self.assertAlmostEqual(converter(373.14999999999998), 100.0)


    def test_convert_from_unit_kelvin_named_from_rankine_named(self):
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(0.0, self.kelvin_named), 0.0)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(255.37222222222221, self.kelvin_named), 459.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(273.14999999999998, self.kelvin_named), 491.66999999999996)
        self.assertAlmostEqual(self.rankine_named.convert_from_unit(373.14999999999998, self.kelvin_named), 671.66999999999996)

    def test_make_converter_kelvin_named_from_rankine_named(self):
        converter = self.kelvin_named.make_converter(self.rankine_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 459.66999999999996)
        self.assertAlmostEqual(converter(273.14999999999998), 491.66999999999996)
        self.assertAlmostEqual(converter(373.14999999999998), 671.66999999999996)


    def test_convert_from_unit_kelvin_named_from_kelvin_named(self):
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(0.0, self.kelvin_named), 0.0)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(255.37222222222221, self.kelvin_named), 255.37222222222221)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(273.14999999999998, self.kelvin_named), 273.14999999999998)
        self.assertAlmostEqual(self.kelvin_named.convert_from_unit(373.14999999999998, self.kelvin_named), 373.14999999999998)

    def test_make_converter_kelvin_named_from_kelvin_named(self):
        converter = self.kelvin_named.make_converter(self.kelvin_named)
        self.assertAlmostEqual(converter(0.0), 0.0)
        self.assertAlmostEqual(converter(255.37222222222221), 255.37222222222221)
        self.assertAlmostEqual(converter(273.14999999999998), 273.14999999999998)
        self.assertAlmostEqual(converter(373.14999999999998), 373.14999999999998)

